package com.jiuqi.xml;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Stack;
import java.io.OutputStream;

public class XMLWriter
{
    private String version;
    private String encoding;
    private String dateFormat;
    private String blank;
    private boolean format;
    private boolean flag;
    private boolean sign;
    private boolean beginLock;
    private boolean attributeLock;
    private boolean valueLock;
    private OutputStream out;
    private Stack tags;
    private int bufferSize;
    private StringBuffer stringBuffer;
    
    public XMLWriter(final OutputStream out) throws XMLException {
        this.version = "1.0";
        this.encoding = "GBK";
        this.dateFormat = "yyyy-MM-dd";
        this.blank = "";
        this.format = true;
        this.flag = false;
        this.sign = false;
        this.beginLock = false;
        this.attributeLock = true;
        this.valueLock = true;
        this.out = null;
        this.tags = new Stack();
        this.bufferSize = 4096;
        this.stringBuffer = new StringBuffer();
        this.out = out;
        this.init();
    }
    
    public XMLWriter(final OutputStream out, final String encoding) throws XMLException {
        this.version = "1.0";
        this.encoding = "GBK";
        this.dateFormat = "yyyy-MM-dd";
        this.blank = "";
        this.format = true;
        this.flag = false;
        this.sign = false;
        this.beginLock = false;
        this.attributeLock = true;
        this.valueLock = true;
        this.out = null;
        this.tags = new Stack();
        this.bufferSize = 4096;
        this.stringBuffer = new StringBuffer();
        this.out = out;
        this.encoding = encoding;
        this.init();
    }
    
    public XMLWriter(final OutputStream out, final String encoding, final String version) throws XMLException {
        this.version = "1.0";
        this.encoding = "GBK";
        this.dateFormat = "yyyy-MM-dd";
        this.blank = "";
        this.format = true;
        this.flag = false;
        this.sign = false;
        this.beginLock = false;
        this.attributeLock = true;
        this.valueLock = true;
        this.out = null;
        this.tags = new Stack();
        this.bufferSize = 4096;
        this.stringBuffer = new StringBuffer();
        this.out = out;
        this.encoding = encoding;
        this.version = version;
        this.init();
    }
    
    private void init() throws XMLException {
        this.write("<?xml version=\"");
        this.write(this.version);
        this.write("\" encoding=\"");
        this.write(this.encoding);
        this.write("\"?");
    }
    
    public void beginNode(final String name) throws XMLException {
        if (this.beginLock) {
            throw new XMLException("开始节点<" + name + ">置于非法位置");
        }
        if (this.format) {
            this.write(">\n");
            this.write(this.blank);
            this.write("<");
            this.write(name);
            this.blank = String.valueOf(this.blank) + "  ";
        }
        else {
            this.write("><");
            this.write(name);
        }
        this.valueLock = false;
        this.attributeLock = false;
        this.flag = true;
        this.sign = false;
        this.tags.push(name);
    }
    
    public void setAttribute(final String name) throws XMLException {
        this.setAttribute(name, "");
    }
    
    public void setAttribute(final String name, final int value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final byte value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final short value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final long value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final float value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final double value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final char value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final boolean value) throws XMLException {
        this.setAttribute(name, String.valueOf(value));
    }
    
    public void setAttribute(final String name, final Date value) throws XMLException {
        final DateFormat df = new SimpleDateFormat(this.dateFormat);
        this.setAttribute(name, df.format(value));
    }
    
    public void setAttribute(final String name, final Object value) throws XMLException {
        this.setAttribute(name, value.toString());
    }
    
    public void setAttribute(final String name, final String value) throws XMLException {
        if (this.attributeLock) {
            throw new XMLException("节点属性 " + name + "=\"" + value + "\" 置于非法位置");
        }
        this.write(" ");
        this.write(name);
        this.write("=\"");
        this.writeXml(value);
        this.write("\"");
        this.beginLock = false;
        this.valueLock = false;
    }
    
    public void setValue() throws XMLException {
        this.setValue("");
    }
    
    public void setValue(final int value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final byte value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final short value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final long value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final float value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final double value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final char value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final boolean value) throws XMLException {
        this.setValue(String.valueOf(value));
    }
    
    public void setValue(final Date value) throws XMLException {
        final DateFormat df = new SimpleDateFormat(this.dateFormat);
        this.setValue(df.format(value));
    }
    
    public void setValue(final Object value) throws XMLException {
        this.setValue(value.toString());
    }
    
    public void setValue(final String value) throws XMLException {
        if (this.valueLock) {
            throw new XMLException("节点内容 " + value + " 置于非法位置");
        }
        this.write(">");
        this.writeXml(value);
        this.beginLock = true;
        this.valueLock = true;
        this.attributeLock = true;
        this.sign = true;
    }
    
    public void setContent(final String content) throws XMLException {
        if (this.valueLock) {
            throw new XMLException("节点内容 " + content + " 置于非法位置");
        }
        this.write(">");
        this.write("<![CDATA[");
        this.write(content);
        this.write("]]>");
        this.beginLock = true;
        this.valueLock = true;
        this.attributeLock = true;
        this.sign = true;
    }
    
    public void writeContentNode(final String name, final String content) throws XMLException {
        this.beginNode(name);
        this.setContent(content);
        this.endNode();
    }
    
    public void endNode() throws XMLException {
        if (this.tags.isEmpty()) {
            throw new XMLException("此结束节点位置不合法，其对应匹配的开始结点未创建");
        }
        if (this.format) {
            this.blank = this.blank.substring(0, this.blank.length() - 2);
        }
        final String name = this.tags.pop();
        if (this.flag) {
            if (this.sign) {
                this.write("</");
                this.write(name);
            }
            else {
                this.write("/");
            }
        }
        else {
            this.write(">\n");
            this.write(this.blank);
            this.write("</");
            this.write(name);
        }
        if (this.tags.empty()) {
            this.writeEnd(">");
        }
        this.flag = false;
        this.valueLock = true;
        this.beginLock = false;
        this.attributeLock = true;
    }
    
    public void writeNode(final String name) throws XMLException {
        this.beginNode(name);
        this.endNode();
    }
    
    public void writeNode(final String name, final String value) throws XMLException {
        this.beginNode(name);
        this.setValue(value);
        this.endNode();
    }
    
    private void write(final String str) throws XMLException {
        if (str == null) {
            throw new XMLException("不允许在XML中写入NULL值");
        }
        this.stringBuffer.append(str);
        if (this.stringBuffer.length() > this.bufferSize) {
            this.flush();
        }
    }
    
    private void writeXml(final String str) throws XMLException {
        if (str == null) {
            throw new XMLException("不允许在XML中写入NULL值");
        }
        for (int i = 0; i < str.length(); ++i) {
            if (str.charAt(i) == '\"') {
                this.write("&quot;");
            }
            else if (str.charAt(i) == '\'') {
                this.write("&apos;");
            }
            else if (str.charAt(i) == '&') {
                this.write("&amp;");
            }
            else if (str.charAt(i) == '<') {
                this.write("&lt;");
            }
            else if (str.charAt(i) == '>') {
                this.write("&gt;");
            }
            else {
                this.write(str.substring(i, i + 1));
            }
        }
    }
    
    private void writeEnd(final String str) throws XMLException {
        this.stringBuffer.append(str);
        this.flush();
    }
    
    public void flush() throws XMLException {
        try {
            this.out.write(this.stringBuffer.toString().getBytes(this.encoding));
            this.stringBuffer.delete(0, this.stringBuffer.length());
        }
        catch (final IOException e) {
            throw new XMLException(e);
        }
    }
    
    public boolean isFormat() {
        return this.format;
    }
    
    public void setFormat(final boolean format) {
        this.format = format;
    }
    
    public String getDateFormat() {
        return this.dateFormat;
    }
    
    public void setDateFormat(final String dateFormat) {
        this.dateFormat = dateFormat;
    }
    
    public int getBufferSize() {
        return this.bufferSize;
    }
    
    public void setBufferSize(final int bufferSize) {
        this.bufferSize = bufferSize;
    }
}
